/*-
 * See the file LICENSE for redistribution information.
 *
 * Copyright (c) 2007,2008 Oracle.  All rights reserved.
 *
 * $Id: ResultsTest.java,v 1.12 2008/04/30 13:02:58 lauren Exp $
 */
package dbxmltest;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import com.sleepycat.dbxml.XmlContainer;
import com.sleepycat.dbxml.XmlDocument;
import com.sleepycat.dbxml.XmlException;
import com.sleepycat.dbxml.XmlManager;
import com.sleepycat.dbxml.XmlQueryContext;
import com.sleepycat.dbxml.XmlResults;
import com.sleepycat.dbxml.XmlTransaction;
import com.sleepycat.dbxml.XmlUpdateContext;
import com.sleepycat.dbxml.XmlValue;

public class ResultsTest {
    private XmlManager mgr = null;
    private XmlContainer cont = null;
    private TestConfig hp;
    private static final String CON_NAME = "testData.dbxml";
    private static String docString = "<?xml version=\"1.0\" ?>"
	+ "<a_node atr1=\"test\" atr2=\"test2\"><b_node/>"
	+ "<c_node>Other text</c_node><d_node/></a_node>";
    private static String docName = "testDoc.xml";

    @BeforeClass
	public static void setupClass() {
	System.out.println("Begin test XmlResults!");
	TestConfig.fileRemove(XmlTestRunner.getEnvironmentPath());
    }

    @Before
	public void setUp() throws Throwable {
	hp = new TestConfig(XmlTestRunner.getEnvironmentType(),
			    XmlTestRunner.isNodeContainer(), XmlTestRunner.getEnvironmentPath());
	mgr = hp.createManager();
	cont = hp.createContainer(CON_NAME, mgr);
    }

    @After
	public void tearDown() throws Throwable {
	hp.closeContainer(cont);
	hp.closeManager(mgr);
	hp.closeEnvironment();
	TestConfig.fileRemove(XmlTestRunner.getEnvironmentPath());
    }

    @AfterClass
	public static void tearDownClass() {
	System.out.println("Finished test XmlResults!");
    }

    /*
     * Test for constructor
     * 
     */
    @Test
	public void testConstructor() throws Throwable {
	XmlTransaction txn = null;
	XmlUpdateContext theContext = mgr.createUpdateContext();
	String myQuery = "collection('testData.dbxml')/a_node/*";
	XmlQueryContext context = mgr.createQueryContext();
	XmlResults results = null;
	XmlResults results_copy = null;

	try {
	    if(hp.isTransactional()){
		txn = mgr.createTransaction();
		cont.putDocument(txn, docName, docString, theContext);
		results = mgr.query(txn, myQuery, context);
	    }else{
		cont.putDocument(docName, docString, theContext);
		results = mgr.query(myQuery, context);
	    }
	    results_copy = new XmlResults(results);
	    assertEquals(results_copy.size(), 3);
	} catch (XmlException e) {
	    if(hp.isTransactional() && txn != null) txn.commit();
	    throw e;
	}finally{
	    if(results_copy != null) results_copy.delete();
	    results_copy = null;
	    if(results != null) results.delete();
	    results = null;
	}

	try {
	    results_copy = new XmlResults(null);
	} catch (XmlException e) {
	    throw e;
	} catch (NullPointerException e) {
	    assertNotNull(e.getMessage());
	}finally{
	    if(results_copy != null) results_copy.delete();
	    results_copy = null;
	    if(hp.isTransactional() && txn != null) txn.commit();
	}
    }

    /*
     * Test for add()
     * 
     */
    @Test
	public void testAdd() throws Throwable {
	XmlTransaction txn = null;
	XmlResults results = mgr.createResults();
	//test add a null object
	XmlValue va = new XmlValue();
	try{
	    results.add(va);
	    fail("Failure in ResultsTest.testAdd()");
	}catch(XmlException e){
	    assertNotNull(e.getMessage());
	    assertEquals(XmlException.INVALID_VALUE, e.getErrorCode());
	}

	XmlValue value = new XmlValue(XmlValue.STRING, "ddd");
	try{
	    results.add(value);
	    assertEquals(results.size(), 1);
	    results.add(value);
	    assertEquals(results.size(), 2);
	    results.add(value);
	    assertEquals(results.size(), 3);
	}catch(XmlException e){
	    throw e;
	}finally{
	    results.delete();
	}

	XmlUpdateContext theContext = mgr.createUpdateContext();
	String myQuery = "collection('testData.dbxml')/a_node/*";
	XmlQueryContext context = mgr.createQueryContext();
	XmlResults results2 = null;
	try{
	    if(hp.isTransactional()){
		txn = mgr.createTransaction();
		cont.putDocument(txn, docName, docString, theContext);
		results2 = mgr.query(txn, myQuery, context);
	    }else{
		cont.putDocument(docName, docString, theContext);
		results2 = mgr.query(myQuery, context);
	    }
	    results2.add(value);
	    assertEquals(results2.size(), 4);
	} catch (XmlException e){
	    if(hp.isTransactional() && txn != null) txn.commit();
	    throw e;
	}finally{
	    if(results2 != null) results2.delete();
	}

	XmlDocument doc = mgr.createDocument();
	doc.setName(docName);
	doc.setContent(docString);
	value = new XmlValue(doc);
	XmlResults results3 = mgr.createResults();
	try{
	    results3.add(value);
	    XmlDocument doc2 = mgr.createDocument();
	    results3.next(doc2);
	    assertEquals(doc2.getContentAsString(), docString);
	    results3.reset();
	    value = results3.next();
	    doc2 = value.asDocument();
	    assertEquals(doc2.getContentAsString(), docString);
	}catch (XmlException e){
	    throw e;
	}finally{
	    results3.delete();
	    if(hp.isTransactional() && txn != null) txn.commit();
	}
    }
    /*
     * Test for delete()
     * 
     */
    @Test
	public void testDelete() throws Throwable {
	XmlResults results = mgr.createResults();
	try{
	    results.delete();
	}catch(Exception e){
	    throw e;
	}
	// after delete
	try {
	    results.size();
	} catch (XmlException e) {
	    assertNotNull(e.getMessage());
	    assertEquals(XmlException.INTERNAL_ERROR, e.getErrorCode());
	}
    }

    /*
     * Test for getEvaluationType()
     * 
     */
    @Test
	public void testGetEvaluationType() throws Throwable {
	XmlResults results = mgr.createResults();
	try {
	    assertEquals(results.getEvaluationType(), XmlQueryContext.Eager);
	} catch (XmlException e) {
	    throw e;
	}finally{
	    results.delete();
	}
    }

    /*
     * Test for hasNext()
     * 
     */
    @Test
	public void testHasNext() throws Throwable {
	XmlTransaction txn = null;
	XmlUpdateContext theContext = mgr.createUpdateContext();
	String myQuery = "collection('testData.dbxml')/a_node/*";
	XmlQueryContext context = mgr.createQueryContext();
	XmlResults results = null;

	boolean bool;
	try {
	    if(hp.isTransactional()){
		txn = mgr.createTransaction();
		cont.putDocument(txn, docName, docString, theContext);
		results = mgr.query(txn, myQuery, context);
	    }else{
		cont.putDocument(docName, docString, theContext);
		results = mgr.query(myQuery, context);
	    }
	    bool = results.hasNext();
	    assertTrue(bool);
	} catch (XmlException e) {
	    if(hp.isTransactional() && txn != null) txn.commit();
	    throw e;
	}finally{
	    if(results != null) results.delete();
	    results = null;
	}
    
	// test results has no value
	myQuery = "collection('testData.dbxml')/a_node/f_node";
	try {
	    if(hp.isTransactional())
		results = mgr.query(txn, myQuery, context);
	    else
		results = mgr.query(myQuery, context);
	    bool = results.hasNext();
	    assertFalse(bool);
	} catch (XmlException e) {
	    if(hp.isTransactional() && txn != null) txn.commit();
	    throw e;
	}finally{
	    if(results != null) results.delete();
	    results = null;
	}
    
	// test in the last of the results
	myQuery = "collection('testData.dbxml')/a_node/d_node";
	try {
	    if(hp.isTransactional())
		results = mgr.query(txn, myQuery, context);
	    else
		results = mgr.query(myQuery, context);
	    results.next();
	    bool = results.hasNext();
	    assertFalse(bool);
	} catch (XmlException e) {
	    throw e;
	}finally{
	    if(results != null) results.delete();
	    results = null;
	    if(hp.isTransactional() && txn != null) txn.commit();
	}
    }

    /*
     * Test for hasPrevious()
     * 
     */
    @Test
	public void testHasPrevious() throws Throwable {
	XmlTransaction txn = null;
	XmlUpdateContext theContext = mgr.createUpdateContext();
	String myQuery = "collection('testData.dbxml')/a_node/*";
	XmlQueryContext context = mgr.createQueryContext();
	XmlResults results = null;
	boolean bool;
	try {
	    if(hp.isTransactional()){
		txn = mgr.createTransaction();
		cont.putDocument(txn, docName, docString, theContext);
		results = mgr.query(txn, myQuery, context);
	    }else{
		cont.putDocument(docName, docString, theContext);
		results = mgr.query(myQuery, context); 
	    }
	    bool = results.hasPrevious();
	    assertFalse(bool);
	    results.next();
	    results.next();
	    bool = results.hasPrevious();
	    assertTrue(bool);
	} catch (XmlException e) {
	    if(hp.isTransactional() && txn != null) txn.commit();
	    throw e;
	}finally{
	    if(results != null) results.delete();
	    results = null;
	}
    
	// test results has no elements
	myQuery = "collection('testData.dbxml')/a_node/f_node";
	try {
	    if(hp.isTransactional())
		results = mgr.query(txn, myQuery, context);
	    else
		results = mgr.query(myQuery, context);
	    bool = results.hasPrevious();
	    assertFalse(bool);
	} catch (XmlException e) {
	    if(hp.isTransactional() && txn != null) txn.commit();
	    throw e;
	}finally{
	    if(results != null) results.delete();
	    results = null;
	}
    
	// only one elements in the results
	myQuery = "collection('testData.dbxml')/a_node/d_node";
	try {
	    if(hp.isTransactional())
		results = mgr.query(txn, myQuery, context);
	    else
		results = mgr.query(myQuery, context);
	    results.next();
	    bool = results.hasPrevious();
	    assertFalse(bool);

	    results.next();
	    bool = results.hasPrevious();
	    assertTrue(bool);

	    results.next();
	    bool = results.hasPrevious();
	    assertTrue(bool);
	} catch (XmlException e) {
	    throw e;
	}finally{
	    if(results != null) results.delete();
	    results = null;
	    if(hp.isTransactional() && txn != null) txn.commit();
	}
    }

    /*
     * Test for isNull()
     * 
     */
    @Test
	public void testIsNull() throws Throwable {
	XmlTransaction txn = null;
	XmlUpdateContext theContext = mgr.createUpdateContext();
	XmlQueryContext context = mgr.createQueryContext();
	XmlResults results = null;
	boolean bool;
	// test empty results
	try {
	    results = mgr.createResults();
	    bool = results.isNull();
	    assertFalse(bool);
	} catch (XmlException e) {
	    throw e;
	}finally{
	    if(results != null) results.delete();
	    results = null;
	}
    
	// test non-empty results
	String myQuery = "collection('testData.dbxml')/a_node/*";
	try {
	    if(hp.isTransactional()){
		txn = mgr.createTransaction();
		cont.putDocument(txn, docName, docString, theContext);
		results = mgr.query(txn, myQuery, context);
	    }else{
		cont.putDocument(docName, docString, theContext);
		results = mgr.query(myQuery, context); 
	    }
	    bool = results.isNull();
	    assertFalse(bool);
	} catch (XmlException e) {
	    throw e;
	}finally{
	    if(results != null) results.delete();
	    results = null;
	    if(hp.isTransactional() && txn != null) txn.commit();
	}
    }

    /*
     * Test for : XmlValue next()
     * 
     */
    @Test
	public void testNext_value() throws Throwable {
	XmlTransaction txn = null;
	XmlUpdateContext theContext = mgr.createUpdateContext();
	String myQuery = "collection('testData.dbxml')/a_node/b_node";
	XmlQueryContext context = mgr.createQueryContext();
	XmlResults results = null;
	XmlValue value = null;

	try {
	    if(hp.isTransactional()){
		txn = mgr.createTransaction();
		cont.putDocument(txn, docName, docString, theContext);
		results = mgr.query(txn, myQuery, context);
	    }else{
		cont.putDocument(docName, docString, theContext);
		results = mgr.query(myQuery, context); 
	    }
	    value = results.next();
	    assertEquals("<b_node/>", value.asString());
	    value = results.next();
	    assertNull(value);
	} catch (XmlException e) {
	    if(hp.isTransactional() && txn != null) txn.commit();
	    throw e;
	}finally{
	    if(results != null) results.delete();
	    results = null;
	}
    
	// test empty results
	myQuery = "collection('testData.dbxml')/a_node/f_node";
	try {
	    if(hp.isTransactional())
		results = mgr.query(txn, myQuery, context);
	    else
		results = mgr.query(myQuery, context);
	    value = results.next();
	    assertNull(value);
	} catch (XmlException e) {
	    if(hp.isTransactional() && txn != null) txn.commit();
	    throw e;
	}finally{
	    if(results != null) results.delete();
	    results = null;
	}
    
	try {
	    results = mgr.createResults();
	    value = results.next();
	    assertNull(value);
	} catch (XmlException e) {
	    throw e;
	}finally{
	    if(results != null) results.delete();
	    results = null;
	    if(hp.isTransactional() && txn != null) txn.commit();
	}
    }

    /*
     * Test for : boolean next(XmlDocument document)
     * 
     */
    @Test
	public void testNext_bool() throws Throwable {
	XmlTransaction txn = null;
	XmlUpdateContext theContext = mgr.createUpdateContext();
	String myQuery = "collection('testData.dbxml')";
	XmlQueryContext context = mgr.createQueryContext();
	XmlResults results = null;
	boolean bool;
	XmlDocument document = mgr.createDocument();

	try {
	    if(hp.isTransactional()){
		txn = mgr.createTransaction();
		cont.putDocument(txn, "testdoc1.xml", docString, theContext);
		cont.putDocument(txn, "testdoc2.xml", docString, theContext);
		cont.putDocument(txn, "testdoc3.xml", docString, theContext);
		results = mgr.query(txn, myQuery, context);
	    }else{
		cont.putDocument("testdoc1.xml", docString, theContext);
		cont.putDocument("testdoc2.xml", docString, theContext);
		cont.putDocument("testdoc3.xml", docString, theContext);
		results = mgr.query(myQuery, context);
	    }
	    bool = results.next(document);
	    assertTrue(bool);
	    assertEquals(document.getName(), "testdoc1.xml");

	    bool = results.next(document);
	    assertTrue(bool);
	    assertEquals(document.getName(), "testdoc2.xml");

	    bool = results.next(document);
	    assertTrue(bool);
	    assertEquals(document.getName(), "testdoc3.xml");

	    // Test in the last place
	    bool = results.next(document);
	    assertFalse(bool);
	} catch (XmlException e) {
	    if(hp.isTransactional() && txn != null) txn.commit();
	    throw e;
	}finally{
	    if(results != null) results.delete();
	    results = null;
	}
	// Test not query as an document
	myQuery = "collection('testData.dbxml')/a_node/f_node";
	XmlDocument doc = mgr.createDocument();
	try {
	    if(hp.isTransactional())
		results = mgr.query(txn, myQuery, context);
	    else
		results = mgr.query(myQuery, context); 
	    bool = results.next(doc);
	    assertFalse(bool);
	} catch (XmlException e) {
	    throw e;
	}finally{
	    if(results != null) results.delete();
	    results = null;
	    if(hp.isTransactional() && txn != null) txn.commit();
	}
    }

    /*
     * Test for: XmlValue peek()
     * 
     */
    @Test
	public void testPeek_value() throws Throwable {
	XmlTransaction txn = null;
	XmlUpdateContext theContext = mgr.createUpdateContext();
	String myQuery = "collection('testData.dbxml')/a_node/*";
	XmlQueryContext context = mgr.createQueryContext();
	XmlResults results = null;
	XmlValue value = null;

	try {
	    if(hp.isTransactional()){
		txn = mgr.createTransaction();
		cont.putDocument(txn, docName, docString, theContext);
		results = mgr.query(txn, myQuery, context);
	    }else{
		cont.putDocument(docName, docString, theContext);
		results = mgr.query(myQuery, context);  
	    }
	    value = results.peek();
	    assertEquals("<b_node/>", value.asString());
	} catch (XmlException e) {
	    if(hp.isTransactional() && txn != null) txn.commit();
	    throw e;
	}finally{
	    if(results != null) results.delete();
	    results = null;
	}
    
	// test empty results
	myQuery = "collection('testData.dbxml')/a_node/f_node";
	try {
	    if(hp.isTransactional())
		results = mgr.query(txn, myQuery, context);
	    else
		results = mgr.query(myQuery, context);
	    value = results.peek();
	    assertNull(value);
	} catch (XmlException e) {
	    if(hp.isTransactional() && txn != null) txn.commit();
	    throw e;
	}finally{
	    if(results != null) results.delete();
	    results = null;
	}
    
	try {
	    results = mgr.createResults();
	    value = results.peek();
	    assertNull(value);
	} catch (XmlException e) {
	    throw e;
	}finally{
	    if(results != null) results.delete();
	    results = null;
	    if(hp.isTransactional() && txn != null) txn.commit();
	}
    }

    /*
     * Test for : boolean peek(XmlDocument document)
     * 
     */
    @Test
	public void testPeek_bool() throws Throwable {
	XmlTransaction txn = null;
	XmlUpdateContext theContext = mgr.createUpdateContext();
	String myQuery = "collection('testData.dbxml')";
	XmlQueryContext context = mgr.createQueryContext();
	XmlResults results = null;
	boolean bool;
	XmlDocument document = mgr.createDocument();

	try {
	    if(hp.isTransactional()){
		txn = mgr.createTransaction();
		cont.putDocument(txn, "testdoc1.xml", docString, theContext);
		cont.putDocument(txn, "testdoc2.xml", docString, theContext);
		cont.putDocument(txn, "testdoc3.xml", docString, theContext);
		results = mgr.query(txn, myQuery, context);
	    }else{
		cont.putDocument("testdoc1.xml", docString, theContext);
		cont.putDocument("testdoc2.xml", docString, theContext);
		cont.putDocument("testdoc3.xml", docString, theContext);
		results = mgr.query(myQuery, context);
	    }
	    bool = results.peek(document);
	    assertTrue(bool);
	    assertEquals(document.getName(), "testdoc1.xml");
	    results.next();
	    results.next();
	    results.next();
	    results.next();
	    // Test in the last place
	    bool = results.peek(document);
	    assertFalse(bool);
	} catch (XmlException e) {
	    if(hp.isTransactional() && txn != null) txn.commit();
	    throw e;
	}finally{
	    if(results != null) results.delete();
	    results = null;
	}
	// Test not query as an document
	myQuery = "collection('testData.dbxml')/a_node/f_node";
	XmlDocument doc = mgr.createDocument();
	try {
	    if(hp.isTransactional())
		results = mgr.query(txn, myQuery, context);
	    else
		results = mgr.query(myQuery, context);
	    bool = results.peek(doc);
	    assertFalse(bool);
	} catch (XmlException e) {
	    throw e;
	}finally{
	    if(results != null) results.delete();
	    results = null;
	    if(hp.isTransactional() && txn != null) txn.commit();
	}
    }

    /*
     * Test for: XmlValue previous()
     * 
     */
    @Test
	public void testPrevious_value() throws Throwable {
	XmlTransaction txn = null;
	XmlUpdateContext theContext = mgr.createUpdateContext();
	String myQuery = "collection('testData.dbxml')/a_node/b_node";
	XmlQueryContext context = mgr.createQueryContext();
	XmlResults results = null;
	XmlValue value = null;

	try {
	    if(hp.isTransactional()){
		txn = mgr.createTransaction();
		cont.putDocument(txn, docName, docString, theContext);
		results = mgr.query(txn, myQuery, context);
	    }else{
		cont.putDocument(docName, docString, theContext);
		results = mgr.query(myQuery, context); 
	    }
	    results.next();
	    results.next();
	    value = results.previous();
	    assertNotNull(value);
	    assertEquals("<b_node/>", value.asString());
	    value = results.previous();
	    assertNull(value);
	} catch (XmlException e) {
	    if(hp.isTransactional() && txn != null) txn.commit();
	    throw e;
	}finally{
	    if(results != null) results.delete();
	    results = null;
	}

	// test empty results
	myQuery = "collection('testData.dbxml')/a_node/f_node";
	try {
	    if(hp.isTransactional())
		results = mgr.query(txn, myQuery, context);
	    else
		results = mgr.query(myQuery, context);
	    value = results.previous();
	    assertNull(value);
	} catch (XmlException e) {
	    if(hp.isTransactional() && txn != null) txn.commit();
	    throw e;
	}finally{
	    if(results != null) results.delete();
	    results = null;
	}
    
	try {
	    results = mgr.createResults();
	    value = results.previous();
	    assertNull(value);
	} catch (XmlException e) {
	    throw e;
	}finally{
	    if(results != null) results.delete();
	    results = null;
	    if(hp.isTransactional() && txn != null) txn.commit();
	}
    }

    /*
     * Test for: boolean previous(XmlDocument document)
     * 
     */
    @Test
	public void testPrevious_bool() throws Throwable {
	XmlTransaction txn = null;
	XmlUpdateContext theContext = mgr.createUpdateContext();
	String myQuery = "collection('testData.dbxml')";
	XmlQueryContext context = mgr.createQueryContext();
	XmlResults results = null;
	boolean bool;
	XmlDocument document = mgr.createDocument();

	try {
	    if(hp.isTransactional()){
		txn = mgr.createTransaction();
		cont.putDocument(txn, "testdoc1.xml", docString, theContext);
		cont.putDocument(txn, "testdoc2.xml", docString, theContext);
		cont.putDocument(txn, "testdoc3.xml", docString, theContext);
		results = mgr.query(txn, myQuery, context);
	    }else{
		cont.putDocument("testdoc1.xml", docString, theContext);
		cont.putDocument("testdoc2.xml", docString, theContext);
		cont.putDocument("testdoc3.xml", docString, theContext);
		results = mgr.query(myQuery, context);
	    }
	    bool = results.previous(document);
	    assertFalse(bool);

	    results.next();
	    bool = results.previous(document);
	    assertFalse(bool);

	    //go to the second will get the first one
	    results.next();
	    bool = results.previous(document);
	    assertTrue(bool);
	    assertEquals(document.getName(), "testdoc1.xml");

	    results.next();
	    results.next();

	    // Test in the last place
	    bool = results.peek(document);
	    assertTrue(bool);
	} catch (XmlException e) {
	    if(hp.isTransactional() && txn != null) txn.commit();
	    throw e;
	}finally{
	    if(results != null) results.delete();
	    results = null;
	}
    
	// Test not query as an document
	myQuery = "collection('testData.dbxml')/a_node/f_node";
	XmlDocument doc = mgr.createDocument();
	try {
	    if(hp.isTransactional())
		results = mgr.query(txn, myQuery, context);
	    else
		results = mgr.query(myQuery, context);
	    bool = results.previous(doc);
	    assertFalse(bool);
	} catch (XmlException e) {
	    throw e;
	}finally{
	    if(results != null) results.delete();
	    results = null;
	    if(hp.isTransactional() && txn != null) txn.commit();
	}
    }

    /*
     * Test for reset()
     * 
     */
    @Test
	public void testReset() throws Throwable {
	XmlTransaction txn = null;
	XmlUpdateContext theContext = mgr.createUpdateContext();
	String myQuery = "collection('testData.dbxml')/a_node/*";
	XmlQueryContext context = mgr.createQueryContext();
	XmlResults results = null;
	XmlValue value = null;
    
	try {
	    if(hp.isTransactional()){
		txn = mgr.createTransaction();
		cont.putDocument(txn, docName, docString, theContext);
		results = mgr.query(txn, myQuery, context);
	    }else{
		cont.putDocument(docName, docString, theContext);
		results = mgr.query(myQuery, context);
	    }
	    value = results.next();
	    assertEquals("<b_node/>", value.asString());
	    results.reset();
	    value = results.next();
	    assertEquals("<b_node/>", value.asString());

	    results.next();
	    results.next();
	    results.next();
	    results.reset();
	    value = results.next();
	    assertEquals("<b_node/>", value.asString());

	    results.reset();
	    results.reset();
	    value = results.next();
	    assertEquals("<b_node/>", value.asString());
	} catch (XmlException e) {
	    throw e;
	}finally{
	    if(results != null) results.delete();
	    results = null;
	    if(hp.isTransactional() && txn != null) txn.commit();
	}
    }

    /*
     * Test for size()
     * 
     */
    @Test
	public void testSize() throws Throwable {
	XmlTransaction txn = null;
	XmlUpdateContext theContext = mgr.createUpdateContext();
	String myQuery = "collection('testData.dbxml')/a_node/*";
	XmlQueryContext context = mgr.createQueryContext();
	XmlResults results = null;

	try {
	    if(hp.isTransactional()){
		txn = mgr.createTransaction();
		cont.putDocument(txn, docName, docString, theContext);
		results = mgr.query(txn, myQuery, context);
	    }else{
		cont.putDocument(docName, docString, theContext);
		results = mgr.query(myQuery, context); 
	    }
	    assertEquals(results.size(), 3);
	} catch (XmlException e) {
	    if(hp.isTransactional() && txn != null) txn.commit();
	    throw e;
	}finally{
	    if(results != null) results.delete();
	    results = null;
	}
    
	myQuery = "collection('testData.dbxml')/a_node/d";
	try {
	    if(hp.isTransactional())
		results = mgr.query(txn, myQuery, context);
	    else
		results = mgr.query(myQuery, context);
	    assertEquals(results.size(), 0);
	} catch (XmlException e) {
	    throw e;
	}finally{
	    if(results != null) results.delete();
	    results = null;
	    if(hp.isTransactional() && txn != null) txn.commit();
	}
    }
}
